## Create empty keyframe where keyframe exists in layers above.
import bpy

class GP_OT_create_empty_frames(bpy.types.Operator):
    bl_idname = "gp.create_empty_frames"
    bl_label = "Create empty frames"
    bl_description = "Create new empty frames on active layer where there is a frame in layer above\n(usefull in color layers to match line frames)"
    bl_options = {'REGISTER','UNDO'}

    @classmethod
    def poll(cls, context):
        return context.active_object is not None and context.active_object.type == 'GPENCIL'

    def execute(self, context):
        obj = context.object
        gpl = obj.data.layers
        gpl.active_index

        ## Only possible on 'fill' layer ??
        # if not 'fill' in gpl.active.info.lower():
            # self.report({'ERROR'}, f"There must be 'fill' text in layer name")
            # return {'CANCELLED'}

        frame_id_list = []
        for i, l in enumerate(gpl):
            # don't list layer below
            if i <= gpl.active_index:
                continue
            # print(l.info, "index:", i)
            for f in l.frames:
                frame_id_list.append(f.frame_number)

        frame_id_list = list(set(frame_id_list))
        frame_id_list.sort()

        current_frames = [f.frame_number for f in gpl.active.frames]

        fct = 0
        for num in frame_id_list:
            if num in current_frames:
                continue
            #Create empty frame
            gpl.active.frames.new(num, active=False)
            fct += 1

        if fct:
            self.report({'INFO'}, f"{fct} frame created on layer {gpl.active.info}")
        else:
            self.report({'WARNING'}, f"No frames to create !")

        return {'FINISHED'}


def register():
    bpy.utils.register_class(GP_OT_create_empty_frames)

def unregister():
    bpy.utils.unregister_class(GP_OT_create_empty_frames)